@@ -7,7 +7,7 @@
  * Authors: Avi Bryant, Jean van Wyk <jeanvanwyk@iname.com>
  * Copyright 2000-2004 Java-GNOME team, all rights reserved. 
  *
- * Special thanks to Avi Bryant who created teh original version (GladeXML)
+ * Special thanks to Avi Bryant who created the original version (GladeXML)
  * for GTK.
  */
 
@@ -17,6 +17,7 @@
 
 #include <glade/glade.h>
 #include <gtk/gtk.h>
+#include <libgtk-java/jg_jnu.h>
 
 #define LIBGLADE_DOMAIN "libglade"
 
@@ -89,31 +90,37 @@ static void xml_connect(const gchar *handler_name, GObject *_source,
     signal = signal_name ? (*env)->NewStringUTF(env, signal_name) : 0L;
     data = signal_data ? (*env)->NewStringUTF(env, signal_data) : 0L;
 
-    (*env)->CallVoidMethod(env, obj, connectMID, handler, _source,
-			   signal, data, _target, after);
+    (*env)->CallVoidMethod(env, obj, connectMID, handler, 
+                           getHandleFromPointer(env, _source),
+			   signal, data, 
+                           getHandleFromPointer(env, _target), after);
     // no need to check for exception since we're returning anyway...
 }
 
 JNIEXPORT jstring JNICALL Java_org_gnu_glade_LibGlade_getWidgetName (
-    JNIEnv *env, jobject o, jint nativepeer)
+    JNIEnv *env, jobject o, jobject nativepeer)
 {
-    GtkWidget* gtk;
-    if (nativepeer == 0)
-	return (jstring)NULL;
-    gtk = (GtkWidget*)(nativepeer);
-    return (*env)->NewStringUTF(env, glade_get_widget_name(gtk));
+    GtkWidget* gtk = (GtkWidget*)getPointerFromHandle(env, nativepeer);
+    if (gtk == NULL) {
+        return (jstring)NULL;
+    }
+    const char* name = glade_get_widget_name(gtk);
+    return (*env)->NewStringUTF(env, name);
 }
 
-JNIEXPORT jint JNICALL Java_org_gnu_glade_LibGlade_getNativeWidget (
+JNIEXPORT jobject JNICALL Java_org_gnu_glade_LibGlade_getNativeWidget (
     JNIEnv *env, jobject o, jstring namestr)
 {
     char *name = (char *)(*env)->GetStringUTFChars(env, namestr, 0);
-    GladeXML* xml = (GladeXML*)(int)(*env)->GetIntField(env, o, gladeFID);
-    jint result;
+    GladeXML* xml = 
+        (GladeXML*)
+        getPointerFromHandle(env, 
+                             (*env)->GetObjectField(env, o, gladeFID));
+    jobject result;
     guint handler_id;
 
     handler_id = set_log_handler(env);
-    result = (jint) glade_xml_get_widget(xml, name);
+    result = getHandleFromPointer(env, glade_xml_get_widget(xml, name));
     remove_log_handler(handler_id);
     
     (*env)->ReleaseStringUTFChars(env, namestr, name);
@@ -152,14 +159,48 @@ Java_org_gnu_glade_LibGlade_glade_1xml_1new_1from_1buffer(
     if ((*env)->ExceptionCheck(env))
 	return;
 	
-    (*env)->SetIntField(env, o, gladeFID, (jint)xml);
+    (*env)->SetObjectField(env, o, gladeFID, getHandleFromPointer(env, xml));
 }
 
+/*
+ * int glade_xml_new(String, String)
+ * Directly call glade_xml_new() with the appropriate filename
+ * so that libglade's internals end up constructing widgets
+ * with path references correctly
+ */
+JNIEXPORT void JNICALL 
+Java_org_gnu_glade_LibGlade_glade_1xml_1new(
+  JNIEnv *env, jobject o, jstring file, jstring root) 
+{
+    GladeXML *xml;
+    // struct jglade_info i;
+    char *filename;
+    char *rootname;
+    guint handler_id;
+    // jthrowable exc;
+
+    filename = file ? (char *)(*env)->GetStringUTFChars(env, file, 0) : NULL;
+    rootname = root ? (char *)(*env)->GetStringUTFChars(env, root, 0) : NULL;
+
+    handler_id = set_log_handler(env);
+    xml = glade_xml_new(filename, rootname, NULL);
+    remove_log_handler(handler_id);
+
+    if ((*env)->ExceptionCheck(env))
+       return;
+       
+    (*env)->SetObjectField(env, o, gladeFID, getHandleFromPointer(env, xml));
+}
+
+
+
 JNIEXPORT void JNICALL
 Java_org_gnu_glade_LibGlade_glade_1xml_1signal_1autoconnect_1full (
     JNIEnv *env, jobject o)
 {
-    GladeXML* xml = (GladeXML*)(int)(*env)->GetIntField(env, o, gladeFID);
+    GladeXML* xml = 
+        (GladeXML*)
+        getPointerFromHandle(env, (*env)->GetObjectField(env, o, gladeFID));
     struct jglade_info i;
     guint handler_id;
 
@@ -174,7 +215,9 @@ Java_org_gnu_glade_LibGlade_glade_1xml_1signal_1autoconnect_1full (
 JNIEXPORT void JNICALL Java_org_gnu_glade_LibGlade_initIDs (
     JNIEnv *env, jclass cls)
 {
-    gladeFID = (*env)->GetFieldID(env, cls, "gladeHandle", "I");
+    gladeFID = (*env)->GetFieldID(env, cls, "gladeHandle", 
+                                  "Lorg/gnu/glib/Handle;");
+
     connectMID = (*env)->GetMethodID(env, cls, "connect",
-	"(Ljava/lang/String;ILjava/lang/String;Ljava/lang/String;IZ)V");
+                                     "(Ljava/lang/String;Lorg/gnu/glib/Handle;Ljava/lang/String;Ljava/lang/String;Lorg/gnu/glib/Handle;Z)V");
 }
